#include "kernel.h"

__global__ void calculate_forces_block_cuda(forces_block_t *forces, const particles_block_t *block1, const particles_block_t *block2, const int block_size)
{
	int id = (blockDim.x * blockIdx.x) + threadIdx.x;
	if (id >= BLOCK_SIZE) return;
	
	float *x = forces->x;
	float *y = forces->y;
	float *z = forces->z;
	
	const int same_block = (block1 == block2);
	const float *pos_x1 = block1->position_x;
	const float *pos_y1 = block1->position_y;
	const float *pos_z1 = block1->position_z;
	const float *mass1  = block1->mass;
	
	const float *pos_x2 = block2->position_x;
	const float *pos_y2 = block2->position_y;
	const float *pos_z2 = block2->position_z;
	const float *mass2  = block2->mass;
	
	float fx = x[id];
	float fy = y[id];
	float fz = z[id];
	
	for (int j = 0; j < BLOCK_SIZE; j++) {
		const float diff_x = pos_x2[j] - pos_x1[id];
		const float diff_y = pos_y2[j] - pos_y1[id];
		const float diff_z = pos_z2[j] - pos_z1[id];
		
		const float distance_squared = diff_x * diff_x + diff_y * diff_y + diff_z * diff_z;
		const float distance = sqrtf(distance_squared);
		
		const float force = (!same_block || distance_squared != 0.0f) ?
			(mass1[id] / (distance_squared * distance)) * (mass2[j] * gravitational_constant) : 0.0f;
		
		fx += force * diff_x;
		fy += force * diff_y;
		fz += force * diff_z;
	}
	x[id] = fx;
	y[id] = fy;
	z[id] = fz;
}
